<?php

namespace Drupal\commerce_dibs\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Exception\SoftDeclineException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;
use Drupal\commerce_price\Price;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "commerce_dibs_easy_hosted_payment_page",
 *   label = "DIBS Easy - Hosted Payment Page",
 *   display_label = @Translation("Pay with DIBS Easy"),
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_dibs\PluginForm\OffsiteRedirect\EasyHostedPaymentPage",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class EasyHostedPaymentPage extends OffsitePaymentGatewayBase implements SupportsAuthorizationsInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'secret_key' => '',
        'terms_path' => '',
        'langcode' => 'en-GB',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#description' => $this->t('You can find the secretKey and checkoutKey for the test and live environment in the Easy administration'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];
    $form['terms_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path to terms and conditions page'),
      '#default_value' => $this->configuration['terms_path'],
      '#required' => TRUE,
    ];
    $form['langcode'] = [
      '#type' => 'select',
      '#title' => $this->t('Language'),
      '#default_value' => $this->configuration['langcode'],
      '#options' => [
        'en-GB' => $this->t('English'),
        'da-DK' => $this->t('Danish'),
        'sv-SE' => $this->t('Swedish'),
        'nb-NO' => $this->t('Norwegian'),
        'de-DE' => $this->t('German'),
        'pl-PL' => $this->t('Polish'),
        'fr-FR' => $this->t('French'),
        'es-ES' => $this->t('Spanish'),
        'nl-NL' => $this->t('Dutch'),
        'fi-FI' => $this->t('Finnish'),
      ],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['secret_key'] = $values['secret_key'];
      $this->configuration['terms_path'] = $values['terms_path'];
      $this->configuration['langcode'] = $values['langcode'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $paymentId = $request->query->get('paymentid');
    /** @var \Drupal\commerce_dibs\EasyApiHelper $apiHelper */
    $apiHelper = \Drupal::service('commerce_dibs.easy_api_helper');
    $apiHelper->setEnvironment($this->configuration['mode']);
    $apiHelper->setSecretKey($this->configuration['secret_key']);
    $dibsPayment = $apiHelper->getPayment($paymentId);
    if (!isset($dibsPayment['payment']['summary']['reservedAmount'])) {
      throw new PaymentGatewayException('Reserved amount property is missing');
    }
    $amount = (new Price($dibsPayment['payment']['summary']['reservedAmount'], $order->getTotalPrice()->getCurrencyCode()))->divide(100);
    $payment = $this->entityTypeManager->getStorage('commerce_payment')->create([
      'state' => 'authorization',
      'amount' => $amount,
      'payment_gateway' => $this->entityId,
      'order_id' => $order->id(),
      'remote_state' => 'OK',
      'remote_id' => $paymentId,
    ]);
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function capturePayment(PaymentInterface $payment, Price $amount = NULL) {

    $this->assertPaymentState($payment, ['authorization']);

    /** @var \Drupal\commerce_dibs\EasyApiHelper $apiHelper */
    $apiHelper = \Drupal::service('commerce_dibs.easy_api_helper');
    $apiHelper->setEnvironment($this->configuration['mode']);
    $apiHelper->setSecretKey($this->configuration['secret_key']);

    $payload = ['amount' => $amount->multiply(100)->getNumber()];
    if (isset($amount) && $payment->getBalance()->compareTo($amount) !== 0) {
      // @todo: Add support for partial capture.
      $payload['orderItems'] = [];
    }
    try {
      $apiHelper->chargesPayment($payment->getRemoteId(), $payload);
    }
    catch (\Exception $exception) {
      throw new SoftDeclineException($exception->getMessage());
    }
    $payment->setCompletedTime(\Drupal::service('datetime.time')->getCurrentTime());
    $payment->setState('completed');
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function voidPayment(PaymentInterface $payment) {

    $this->assertPaymentState($payment, ['authorization']);

    /** @var \Drupal\commerce_dibs\EasyApiHelper $apiHelper */
    $apiHelper = \Drupal::service('commerce_dibs.easy_api_helper');
    $apiHelper->setEnvironment($this->configuration['mode']);
    $apiHelper->setSecretKey($this->configuration['secret_key']);

    $payload = ['amount' => $payment->getAmount()->multiply(100)->getNumber()];
    try {
      $apiHelper->cancelPayment($payment->getRemoteId(), $payload);
    }
    catch (\Exception $exception) {
      throw new SoftDeclineException($exception->getMessage());
    }
    $payment->setState('authorization_voided');
    $payment->save();
  }

}
